using System;
using System.Collections;
using Microsoft.SharePoint.Administration;
using Microsoft.Office.Server;
using Microsoft.Office.Server.Administration;
using Microsoft.Office.Server.UserProfiles;
using Microsoft.SharePoint;
using System.Diagnostics;
using TechEd09.SharePoint.Diagnostics;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using Microsoft.SharePoint.Utilities;

namespace TechEd09.SharePoint.UserProfileWatch
{
    class UserProfSync
    {
        //TODO : Pull these from Persisted Settings
        //protected const string ProfileFields = "Fax,CellPhone,AboutMe";

        private Hashtable changedUsers;
        private SPWebApplication webApp;
        
        private int _changeAge = 30;

        private string ProfileFields = string.Empty;

        public UserProfSync(string changeFields, int changeAge, SPWebApplication webApplication)
        {
            changedUsers = new Hashtable();
            _changeAge = changeAge;
            webApp = webApplication;
            ProfileFields = changeFields;
            Debug.WriteLine(String.Format("TechEd09 Profile Sync Timer Job : Sync Job Fields: {0}", ProfileFields));
            TraceLogger.LogMessage(String.Format("TechEd09 Profile Sync Timer Job : Sync Job Fields: {0}", ProfileFields));
        }

        public void RetrieveUserProfileChanges()
        {
            Debug.WriteLine("TechEd09 Profile Sync Timer Job: Starting Retrieve");
            TraceLogger.LogMessage("TechEd09 Profile Sync Timer Job: Starting Retrieve");

            UserProfileManager profileManager = new UserProfileManager(ServerContext.GetContext(this.webApp));

            DateTime tokenStart = DateTime.UtcNow.Subtract(TimeSpan.FromMinutes(_changeAge));
            UserProfileChangeToken changeToken = new UserProfileChangeToken(tokenStart);

            UserProfileChangeQuery changeQuery = new UserProfileChangeQuery(false, true);
            changeQuery.ChangeTokenStart = changeToken;
            changeQuery.UserProfile = true;
            changeQuery.SingleValueProperty = true;
            changeQuery.MultiValueProperty = true;

            UserProfileChangeCollection changes = profileManager.GetChanges(changeQuery);

            Debug.WriteLine(String.Format("User Profile Sync found {0} changes.", changes.Count.ToString()));
            TraceLogger.LogMessage(String.Format("User Profile Sync found {0} changes.", changes.Count.ToString()));

            foreach (UserProfileChange change in changes)
            {
                if (change is UserProfilePropertyValueChange)
                {
                    UserProfilePropertyValueChange propertyChange = (UserProfilePropertyValueChange)change;

                    //Only need this if I need to pass a hash table of users
                    if (changedUsers.Contains(propertyChange.AccountName))
                        continue;

                    if (ProfileFields.Contains(propertyChange.ProfileProperty.Name))
                    {
                        Debug.WriteLine(String.Format("--> User {0} Property: {1} type {2} value {3}",
                            propertyChange.AccountName, propertyChange.ProfileProperty.Name, propertyChange.ChangeType.ToString(),
                            propertyChange.ChangedUser[propertyChange.ProfileProperty.Name.ToString()].Value.ToString()));

                        TraceLogger.LogMessage(String.Format("--> User {0} Property: {1} type {2} value {3}",
                            propertyChange.AccountName, propertyChange.ProfileProperty.Name, propertyChange.ChangeType.ToString(),
                            propertyChange.ChangedUser[propertyChange.ProfileProperty.Name.ToString()].Value.ToString()));


                        // Add the user profile to a hash table                           
                        changedUsers.Add(propertyChange.AccountName, propertyChange.ChangedUser);
                    }
                }
            }

            Debug.WriteLine("TechEd09 Profile Sync Timer Job: Done with Retrieve");
            TraceLogger.LogMessage("TechEd09 Profile Sync Timer Job : Done with retrieve.");
        }

        public void UpdateImage()
        {
            Image img;
            int MaxWidth = 100;
            int MaxHeight = 100;
            string fileName = String.Empty;
 
            try
            {
                TraceLogger.LogMessage(String.Format("TechEd09 Profile Sync Timer Job: Updating image: {0}", this.ToString()));
                //Process each entry
                foreach (DictionaryEntry profile in changedUsers)
                {
                    Debug.WriteLine(String.Format("TechEd09 Profile Sync Timer Job Update User: {0}", profile.Key));
                    TraceLogger.LogMessage(String.Format("TechEd09 Profile Sync Timer Job Update User: {0}", profile.Key));
                    //string xmlParam = ProcessXML(this.ProfileFields, (UserProfile)profile.Value);

                    UserProfile uProfile = (UserProfile)profile.Value;
                    //TODO : Get this from a global
                    try
                    {
                        //TODO : make this a variable with the rest of the code.
                        fileName = uProfile["PictureURL"].Value.ToString();
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine("Filename is empty");
                        Debug.WriteLine(ex.Message);
                    }

                    //Bail if they removed the URL.
                    if (fileName == String.Empty)
                    {
                        Debug.WriteLine("Filename is empty");
                        return;
                    }

                    try
                    {
                        Debug.WriteLine("Process the file");
                        //Get the Personal site info and see if the file is there
                        using (SPSite uSite = uProfile.PersonalSite)
                        {
                            using (SPWeb web = uSite.RootWeb)
                            {

                                // !!! This is an assumption, it is possible that the file resides outside the users library.
                                SPFile file = web.GetFile(fileName);

                                //Execute the code
                                img = Image.FromStream(file.OpenBinaryStream());
                                //TODO : Get these from a global size and field name

                                if ((img.Height >= MaxHeight) || (img.Width >= MaxWidth))
                                {
                                    img = resizeImage(img, new Size(MaxWidth, MaxHeight));
                                    RenameImage(img, file, uProfile);
                                }
                                //TODO : Add a notification here!

                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine("TechEd09 Profile Sync Timer Job: Exception");
                        Debug.WriteLine(ex.Message);
                        TraceLogger.LogMessage("TechEd09 Profile Sync Timer Job : Exception: ", ex);
                        
                    }
                }
                

                Debug.WriteLine("TechEd09 Profile Image Update Timer Job Complete");
                TraceLogger.LogMessage("TechEd09 Profile Image Update Job Complete");
            }
            catch (Exception ex)
            {
                Debug.WriteLine(String.Format("Error updating image: {0}", ex.Message));
                TraceLogger.LogMessage(String.Format("Error updating image: {0}", ex.Message), TraceLogging.TraceSeverity.CriticalEvent);
            }
        }

        public void RenameImage(Image image, SPFile file, UserProfile profile)
        {
            Debug.WriteLine(String.Format("Rename the file {0} to {1}", file.Name, profile.ID));
            //Write the file back to the library
            
            Debug.WriteLine("Server Rel: " + file.ParentFolder.ServerRelativeUrl);
            Debug.WriteLine("ParentFolder: " + file.ParentFolder.Url);

            //Create a new SPFile and Add the image to it then save it back to the library
            SPFolder oFolder = file.ParentFolder;
            
            //Stream stream = new ;
            using (MemoryStream memStream = new MemoryStream())
            {
                Debug.WriteLine("Profile.Name: " + profile["PreferredName"].Value.ToString());
                //Hashtable metadata = new Hashtable();
                //metadata.Add("vti_title", profile["PreferredName"].Value.ToString());
                //metadata.Add("Description", "System generated profile image of standard size. Do not delete.");
                
                //Hashtbl["Description"] = "System generated profile image of standard size. Do not delete.";
                //Hashtbl["vti_title"] = profile["PreferredName"].Value.ToString();


                //TODO : Add Quality params
                image.Save(memStream, System.Drawing.Imaging.ImageFormat.Jpeg);
                Debug.WriteLine(String.Format("Write {0}{1}",profile.ID,".jpg"));
                string newFile = String.Format("{0}{1}", profile.ID, ".jpg");


                //SPFile newProfPic = oFolder.Files.Add(newFile, memStream, metadata, true);
                SPFile newProfPic = oFolder.Files.Add(newFile, memStream, true);
                SPListItem item = newProfPic.Item;
                item["Title"] = profile["PreferredName"].Value.ToString();
                item["Description"] = "System generated profile image of standard size. Do not delete.";
                item.Update();
                //oFolder.Update();
                Debug.WriteLine("Holy Cow....it worked!");
                //Update the PictureURL property
                Debug.WriteLine("Update the Profile: " + SPUtility.GetFullUrl(file.Item.Web.Site, newProfPic.ServerRelativeUrl));
                profile["PictureURL"].Value = SPUtility.GetFullUrl(file.Item.Web.Site, newProfPic.ServerRelativeUrl);
                
                //TODO : Try-Catch
                profile.Commit();
            }
        }

        private Image resizeImage(Image imgToResize, Size size)
        {
            int sourceWidth = imgToResize.Width;
            int sourceHeight = imgToResize.Height;

            float nPercent = 0;
            float nPercentW = 0;
            float nPercentH = 0;

            nPercentW = ((float)size.Width / (float)sourceWidth);
            nPercentH = ((float)size.Height / (float)sourceHeight);

            if (nPercentH < nPercentW)
                nPercent = nPercentH;
            else
                nPercent = nPercentW;

            int destWidth = (int)(sourceWidth * nPercent);
            int destHeight = (int)(sourceHeight * nPercent);

            Bitmap b = new Bitmap(destWidth, destHeight);
            Graphics g = Graphics.FromImage((Image)b);
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;

            g.DrawImage(imgToResize, 0, 0, destWidth, destHeight);
            g.Dispose();

            return (Image)b;
        }
    }
}
