using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using TechEd09.SharePoint.Diagnostics;
using System.Diagnostics;

namespace TechEd09.SharePoint.UserProfileWatch
{
    public class UserProfSyncJob : SPJobDefinition
    {
        public UserProfSyncJob()
            : base()
        {
        }

        public UserProfSyncJob(string jobName, SPService service, SPServer server, SPJobLockType targetType)
            : base(jobName, service, server, targetType)
        {
        }

        public UserProfSyncJob(string jobName, SPWebApplication webApplication)
            : base(jobName, webApplication, null, SPJobLockType.Job)
        {
            this.Title = jobName;
            Debug.WriteLine(String.Format("Created TechEd09 Profile Sync Timer Job: {0}",jobName));
            TraceLogger.LogMessage(String.Format("Created TechEd09 Profile Sync Timer Job: {0}", jobName));
        }

        public override void Execute(Guid targetInstanceId)
        {
            string profileFields = Globals.TechEd09_ProfileJobFields;
            int queryInterval = 30;

            Debug.WriteLine("TechEd09 Profile Sync Timer Job: Begin Execute.");
            TraceLogger.LogMessage("TechEd09 Profile Sync Timer Job: Begin Execute.");

            //#if (DEBUG)
            //    System.Diagnostics.Trace.Assert(false);
            //#endif

            try
            {
                //Get the properties from the Timer Job
                profileFields = this.Properties[Globals.TechEd09_ProfileJobFields].ToString();
                queryInterval = (this.Schedule as SPMinuteSchedule).Interval;
                
                Debug.WriteLine(String.Format("TechEd09 Profile Sync Timer Job, Fields: {0}, Interval {1}", profileFields, queryInterval.ToString()));
                TraceLogger.LogMessage(String.Format("TechEd09 Profile Sync Timer Job, Fields: {0}, Interval {1}", profileFields, queryInterval.ToString()));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error gettting Job Properties");
                TraceLogger.LogMessage("Error Getting Profile Sync Job Properties.", TraceLogging.TraceSeverity.CriticalEvent);
                TraceLogger.LogMessage(ex.Message, TraceLogging.TraceSeverity.CriticalEvent);
            }

            //Create the Sync
            UserProfSync userProfSync = new UserProfSync(profileFields, queryInterval, this.WebApplication);
            
            userProfSync.RetrieveUserProfileChanges();
            //TODO : Run Update Here
            userProfSync.UpdateImage();

            
        }

    }
}
