﻿using System;
using System.Collections.Specialized;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.StsAdmin;

namespace TechEd09.SharePoint.UserProfileWatch
{
    public class StsAdm : ISPStsadmCommand
    {
        public string GetHelpMessage(string command)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nManually runs the TechEd09 User Profile Photo Update process.\r\n\r\nParameters:");
            sb.Append("\r\n\r\n-url <url to central administration>");
            sb.Append("\r\n\t[-queryinterval <change log query in minutes. Default is 60 minutes.>");
            
            return sb.ToString();
        }

        public int Run(string command, StringDictionary keyValues, out string output)
        {
            command = command.ToLowerInvariant();

            switch (command)
            {
                case "teched09-runprofileupdate":
                    return this.ExecuteProfileSync(keyValues, out output);

                default:
                    throw new InvalidOperationException();
            }
        }

        private int ExecuteProfileSync(StringDictionary keyValues, out string output)
        {
            int queryInt = 60;
            string fields = Globals.TechEd09_ProfileFields;
            
            StringBuilder sb = new StringBuilder();

            if (!keyValues.ContainsKey("url"))
                throw new Exception("url parameter is required");

            sb.AppendLine("Running TechEd09 User Profile Watcher");
            
            if (keyValues.ContainsKey("queryinterval"))
                queryInt = Convert.ToInt32(keyValues["queryinterval"].ToString());
            else
                sb.AppendFormat("Using default query interval: {}", queryInt.ToString());

            //if (keyValues.ContainsKey("profilefields"))
            //    fields = keyValues["profilefields"].ToString();
            //else
            //    sb.AppendFormat("Using default field list: {0}", fields);


            using (SPSite site = new SPSite(keyValues["url"].ToString()))
            {

                UserProfSync userProfSync = new UserProfSync(fields, queryInt, site.WebApplication);
                userProfSync.RetrieveUserProfileChanges();
                //TODO : Run Update Here
                userProfSync.UpdateImage();
            }
            
            output = sb.ToString();
            return 0;

        }
    }
}
